﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;

namespace HeuristicLab.Analysis.FitnessLandscape {
  [Item("Random Walk Calculator", "Calculates characteristics from a random walk.")]
  [StorableClass]
  public class RandomWalkCalculator : NamedItem, ICharacteristicCalculator {

    [Storable]
    private IProblem problem;
    public IProblem Problem {
      get { return problem; }
      set {
        if (problem == value) return;
        problem = value;
        var soProblem = problem as ISingleObjectiveHeuristicOptimizationProblem;
        walker.Problem = soProblem;
      }
    }

    [Storable]
    private RandomWalk walker;

    [StorableConstructor]
    private RandomWalkCalculator(bool deserializing) : base(deserializing) { }
    private RandomWalkCalculator(RandomWalkCalculator original, Cloner cloner)
      : base(original, cloner) {
      problem = cloner.Clone(original.problem);
      walker = cloner.Clone(original.walker);
      characteristics = cloner.Clone(original.characteristics);
    }
    public RandomWalkCalculator() : this(new RandomWalk()) { }
    public RandomWalkCalculator(RandomWalk walker) {
      Name = ItemName;
      Description = ItemDescription;
      this.walker = walker;
      characteristics = new CheckedItemList<StringValue>(
        new[] { "AutoCorrelation1", "CorrelationLength", "InformationContent",
        "PartialInformationContent", "DensityBasinInformation", "InformationStability",
        "Diversity", "Regularity", "TotalEntropy", "PeakInformationContent",
        "PeakDensityBasinInformation" }.Select(x => new StringValue(x)));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RandomWalkCalculator(this, cloner);
    }

    private CheckedItemList<StringValue> characteristics;
    public ReadOnlyCheckedItemList<StringValue> Characteristics {
      get { return characteristics.AsReadOnly(); }
    }

    public bool CanCalculate() {
      return Problem is ISingleObjectiveHeuristicOptimizationProblem
        && Problem.Operators.Any(x => x is IManipulator);
    }

    public IEnumerable<IResult> Calculate() {
      walker.Prepare(true);
      using (var waitHandle = new AutoResetEvent(false)) {
        EventHandler evHandle = (sender, e) => {
          if (walker.ExecutionState == ExecutionState.Paused
          || walker.ExecutionState == ExecutionState.Stopped) waitHandle.Set();
        };
        walker.ExecutionStateChanged += evHandle;
        try {
          walker.Start();
          waitHandle.WaitOne();
        } finally { walker.ExecutionStateChanged -= evHandle; }
      }
      foreach (var p in characteristics.CheckedItems) {
        var resultName = "RandomWalk." + walker.MutatorParameter.Value.Name + "." + p.Value.Value;
        IResult result;
        if (walker.Results.TryGetValue(p.Value.Value, out result)) {
          yield return new Result(resultName, result.Value);
        } else yield return new Result(resultName, new DoubleValue(0));
      }
      walker.Prepare(true);
    }

    public void CollectParameterValues(IDictionary<string, IItem> values) {
      walker.CollectParameterValues(values);
    }

    public IKeyedItemCollection<string, IParameter> Parameters {
      get { return ((IParameterizedItem)walker).Parameters; }
    }
  }
}
